Package: ggpower
Title: Publication-Ready Power Analysis and Visualization
Version: 0.1.2
Authors@R: 
    person("Yaoxiang", "Li", , "liyaoxiang@outlook.com", role = c("aut", "cre"))
Author: Yaoxiang Li [aut, cre]
Maintainer: Yaoxiang Li <liyaoxiang@outlook.com>
Description: Provides statistical power analysis and sample size calculations
    for t-tests, ANOVA, regression, chi-square, proportion, correlation,
    nonparametric, biomarker, and clinical trial designs. Includes a
    scriptable API via power_compute(), publication-ready 'ggplot2'
    visualizations, and an optional 'Shiny' application.
URL: https://github.com/YaoxiangLi/ggpower,
        https://yaoxiangli.github.io/ggpower/
BugReports: https://github.com/YaoxiangLi/ggpower/issues
License: GPL (>= 3)
Imports: bs4Dash, config, ggplot2, golem, shiny
Encoding: UTF-8
RoxygenNote: 7.3.2
Suggests: knitr, pkgdown, rmarkdown, spelling, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Language: en-US
NeedsCompilation: no
Packaged: 2026-07-03 19:58:15 UTC; Li
Repository: CRAN
Date/Publication: 2026-07-10 20:50:17 UTC
