# Generated by roxygen2: do not edit by hand

export(format_result_html)
export(ggpower_t_one_sample)
export(ggpower_ttest)
export(ggpower_result)
export(ggpower_calculator)
export(ggpower_tests)
export(plot_distribution)
export(plot_power_curve)
export(effect_size_d)
export(effect_size_f)
export(effect_size_f2)
export(effect_size_f2_increase)
export(effect_size_h)
export(effect_size_q)
export(effect_size_w)
export(eta2_from_f)
export(odds_ratio_from_probs)
export(power_t_one_sample)
export(power_compute)
export(power_t_paired)
export(power_t_two_sample)
export(r2_from_f2)
export(run_app)
export(save_distribution_plot)
export(save_power_plot)
export(theme_ggpower)
S3method(print,ggpower_result)
import(shiny)
importFrom(bs4Dash,bs4DashBody)
importFrom(bs4Dash,bs4DashControlbar)
importFrom(bs4Dash,bs4DashFooter)
importFrom(bs4Dash,bs4DashNavbar)
importFrom(bs4Dash,bs4DashPage)
importFrom(bs4Dash,bs4DashSidebar)
importFrom(bs4Dash,bs4TabItem)
importFrom(bs4Dash,bs4TabItems)
importFrom(bs4Dash,menuItem)
importFrom(bs4Dash,sidebarMenu)
importFrom(golem,activate_js)
importFrom(golem,add_resource_path)
importFrom(golem,bundle_resources)
importFrom(golem,favicon)
importFrom(golem,with_golem_options)
importFrom(shiny,NS)
importFrom(shiny,div)
importFrom(shiny,icon)
importFrom(shiny,shinyApp)
importFrom(shiny,tagList)
importFrom(shiny,tags)
importFrom(stats,pt)
importFrom(stats,qt)
importFrom(utils,capture.output)
