## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggpower)

## ----matrix-------------------------------------------------------------------
tests <- ggpower_tests()
knitr::kable(
  tests[, c("id", "module", "domain", "family", "parity", "method", "modes")],
  row.names = FALSE
)

## ----counts-------------------------------------------------------------------
as.data.frame(table(tests$module))

## ----links, echo=FALSE--------------------------------------------------------
vignette_map <- data.frame(
  module = c(
    rep("getting started", 3), rep("workspace", 2), rep("biomarker", 1),
    rep("clinical", 1), rep("reference", 3)
  ),
  article = c(
    "choosing-a-power-analysis", "getting-started-gui", "scenario-guide",
    "t-tests", "workspace-test-families",
    "biomarker-endpoints", "clinical-trials",
    "analysis-modes", "support-matrix", "reference-validation"
  ),
  stringsAsFactors = FALSE
)
knitr::kable(vignette_map, col.names = c("Module area", "Vignette slug"))

