\name{theme_ggpower}
\alias{theme_ggpower}
\title{Publication-ready ggpower theme}
\usage{
theme_ggpower(base_size = 12, base_family = "")
}
\arguments{
\item{base_size}{Base font size.}
\item{base_family}{Base font family.}
}
\value{
A ggplot2 theme.
}
\description{
Provides consistent typography, spacing, and grid styling for ggpower figures.
}
