% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_ttest.R
\name{ggpower_ttest}
\alias{ggpower_ttest}
\title{Plot Power Curve for a Two-Sample t-Test}
\usage{
ggpower_ttest(d, alpha = 0.05, n_range = seq(10, 100, by = 5),
  tails = "two")
}
\arguments{
\item{d}{Numeric. The effect size (Cohen's d).}

\item{alpha}{Numeric. The significance level (default 0.05).}

\item{n_range}{Numeric vector. A vector of sample sizes per group (default is seq(10, 100, by = 5)).}

\item{tails}{Character. \code{"two"} or \code{"one"}.}
}
\value{
A ggplot object showing the power curve.
}
\description{
This function creates a ggplot2 power curve for a two-sample t test.
}
\examples{
# Create a power curve for d = 0.5 over a range of sample sizes per group
ggpower_ttest(d = 0.5, alpha = 0.05, n_range = seq(10, 100, by = 5))
}
