% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power_registry.R
\name{ggpower_tests}
\alias{ggpower_tests}
\title{List supported statistical power tests}
\usage{
ggpower_tests(domain = NULL, module = NULL)
}
\arguments{
\item{domain}{Optional character vector to filter by domain (\code{general},
\code{biomarker}, \code{pharma}).}

\item{module}{Optional character vector to filter by app module (\code{workspace},
\code{biomarker}, \code{clinical}).}
}
\value{
A data frame describing tests available to \code{power_compute()}.
}
\description{
Lists the tests available to \code{power_compute()}.
}
\examples{
ggpower_tests()
ggpower_tests(module = "biomarker")
}
